from django.db import models
from userprofiles.models import UserProfile
from teams.models import *
from django.utils.translation import ugettext_lazy as _
from common.shortcuts import queryset_sum, queryset_count

import datetime

class BaseMeeting(models.Model):
    """
    a simple basic meeting
    """
    name = models.CharField(max_length=150, verbose_name=_('Meeting Name'))
    date_begin = models.DateTimeField(help_text=_('start date and local time of the meeting in UTC'), verbose_name=_('Begins'), db_index=True)
    date_end = models.DateTimeField(help_text=_('end date and local time of the meeting in UTC'), verbose_name=_('Ends'), db_index=True)
    channel = models.CharField(max_length=150, verbose_name=_('Meeting Channel'), help_text=_('Channel that the meeting will be held in. Include #'), null=True)
    logs = models.URLField(verbose_name=_('Meeting Log'), help_text=_('After the meeting, add a link to the meeting log'), max_length=200, verify_exists=False, blank=True, null=True)
    minutes = models.URLField(verbose_name=_('Meeting Minutes'), help_text=_('After the meeting, add a link to the meeting minutes'), max_length=200, verify_exists=False, blank=True, null=True)
    date_created = models.DateTimeField(help_text=_('the date and time when the event was created'), default=datetime.datetime.now(), db_index=True)
    chair = models.ForeignKey(UserProfile, verbose_name=_('Meeting Chair'), blank=True, null=True)

    def __unicode__(self):
        return self.name
    
    def as_ical(self, cal):
        """
        return a event as ical
        """
        event = cal.add('vevent')
        event.add('uid').value = str(self.id)
        event.add('dtstart').value = self.date_begin
        event.add('dtend').value = self.date_end
        event.add('categories').value = ['Ubuntu Team Meeting']
        event.add('summary').value = self.name or ''

    def is_past(self):
        return self.date_end < datetime.datetime.today()

class TeamMeetingManager(models.Manager):
    """ manager for a team event """

    def next_meetings(self):
        """ a list with all upcoming team meetings """
        return self.filter(date_end__gt=datetime.datetime.now()).order_by('date_end')

    def history_meetings(self):
        """ all team meetings in history """
        return self.filter(date_end__lt=datetime.datetime.now()).order_by('-date_end')

class TeamMeeting(BaseMeeting):
    """
    a meeting of one or more teams
    """
    teams = models.ManyToManyField(Team, help_text=_('teams of this meeting.'), verbose_name=_('Teams Participating'), db_index=True)
    template = 'meetings/team_meeting_li.inc.html'
        
    def __unicode__(self):
        return "%s %s" % (self.name, self.date_begin or "")

    @models.permalink
    def get_absolute_url(self):
        return ('team-meeting-detail', [str(self.id)])
        
    #add manager
    objects = TeamMeetingManager()

class AgendaItemNode(object):

    def __init__(self, item):
        self.item = item
        self.parent = None
        self.children = []
        
    def as_ol(self):
        if len(self.children) > 0:
            return '<li>%s - %s</li>\n<ol>\n%s</ol>\n' % (self.item.title, self.item.sig, '\n'.join([c.as_ol() for c in self.children]))
        else:
            return '<li>%s - %s</li>\n' % (self.item.title, self.item.sig)
        
    def as_ul(self):
        if len(self.children) > 0:
            return '<li>%s - %s</li>\n<ul>\n%s</ul>\n' % (self.item.title, self.item.sig, '\n'.join([c.as_ul() for c in self.children]))
        else:
            return '<li>%s - %s</li>\n' % (self.item.title, self.item.sig)
        
    def __unicode__(self):
        return self.item.title
        
class AgendaItemManager(models.Manager):

    def top(self):
        return self.filter(parent__isnull=True).order_by('order')
            
    def as_tree(self):
        cache = {}
        tree = []
        all_items = self.order_by('parent__id', 'order')
        for i in all_items:
            node = AgendaItemNode(i)
            cache[i.pk] = node
            if i.parent is None or i.parent.pk == 0:
                tree.append(node)
            else:
                node.parent = cache[i.parent.pk]
                node.parent.children.append(node)
        return tree
            
class AgendaItem(models.Model):

    class Meta:
        ordering = ('parent__id', 'order', 'created_date')
        
    meeting = models.ForeignKey(BaseMeeting, verbose_name=_('Meeting'), related_name='agenda', help_text=_('meeting during which this agenda item is to be discussed'))
    title = models.CharField(verbose_name=_('Title'), max_length = 150, help_text=_('descriptive name for this item'))
    owner = models.ForeignKey(UserProfile, verbose_name=_('Owner'), help_text=_('person proposing or responsible for this item'))
    created_date = models.DateTimeField(verbose_name=_('Created Date'), auto_now_add=True, help_text=_('timestamp of when this item was created'))
    description = models.TextField(verbose_name=_('Description'), help_text=_('detailed description of this item'), blank=True, null=True)
    parent = models.ForeignKey('self', verbose_name=_('Parent Agenda Item'), related_name='children', help_text=_('agenda item that contains this item'), blank=True, null=True)
    order = models.PositiveIntegerField(verbose_name=_('Order'), help_text=_('index number of where this item falls in the agenda'), default=1)
    log = models.URLField(verbose_name=_('Log URL'), max_length=200, verify_exists=False, help_text=_('URL to this item\'s discussion'), blank=True, null=True)
    
    objects = AgendaItemManager()
    
    def get_descendents(self):
        descendents = []
        if self.id is None:
            return descendents
        for child in self.children.all():
            descendents.append(child)
            descendents.extend(child.descendents)
        return descendents
    descendents = property(get_descendents)
    
    def get_sig(self):
        return '<a href="http://launchpad.net/~%s" target="launchpaduser">%s</a> %s' % (self.owner.user.username, self.owner.realname, self.created_date)
        
    sig = property(get_sig)
    
    def save(self, *args, **kargs):
        if not self.created_date:
            self.created_date = datetime.datetime.now()
        return super(AgendaItem, self).save(*args, **kargs)    
    
    def __unicode__(self):
        if self.parent is None:
            return '%s' % self.title
        else:
            return '%s->%s' % (self.parent, self.title)
    
